@@ -4,25 +4,25 @@
 # License AGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html)
 
 
+import ast
 import os
 import signal
 import sys
 import threading
+from pathlib import Path
+from typing import Optional
 
 import _pytest
 import _pytest.python
-import py.code
-import py.error
-import py.path
 import pytest
 
-try:
-    import openerp
-    odoo = openerp
-    odoo_namespace = 'openerp'
-except ImportError:  # Odoo >= 10.0
-    import odoo  # noqa
-    odoo_namespace = 'odoo'
+from _pytest._code.code import ExceptionInfo
+import _pytest._py.error as error
+
+from pathlib import Path
+
+import odoo
+import odoo.tests
 
 
 def pytest_addoption(parser):
@@ -89,7 +89,7 @@ def load_registry():
     # And also give wrong timing indications.
     # Finally we enable `testing` flag on current thread
     # since Odoo sets it when loading test suites.
-    threading.currentThread().testing = True
+    threading.current_thread().testing = True
     odoo.registry(odoo.tests.common.get_db_name())
 
 
@@ -124,17 +124,12 @@ class OdooTestModule(_pytest.python.Module):
             if names[-1] == "__init__":
                 names.pop()
             modname = ".".join(names)
-            # modules installed with pip for odoo<=9 are into the
-            # odoo_addons namespace. Makes module part of the odoo namespace
-            if modname.startswith('odoo_addons'):
-                modname = odoo_namespace + '.addons.' + modname.replace(
-                    'odoo_addons.', '')
-            # for modules in openerp/addons, since there is a __init__ the
+            # for modules in odoo/addons, since there is a __init__ the
             # module name is already fully qualified (maybe?)
-            if (not modname.startswith(odoo_namespace + '.addons.')
-                    and modname != odoo_namespace + '.addons'
-                    and modname != odoo_namespace):
-                modname = odoo_namespace + '.addons.' + modname
+            if (not modname.startswith('odoo.addons.')
+                    and modname != 'odoo.addons'
+                    and modname != 'odoo'):
+                modname = 'odoo.addons.' + modname
 
             __import__(modname)
             mod = sys.modules[modname]
@@ -152,13 +147,14 @@ class OdooTestModule(_pytest.python.Module):
                     modfile = modfile[:-12]
             try:
                 issame = self.fspath.samefile(modfile)
-            except py.error.ENOENT:
+            except error.ENOENT:
                 issame = False
             if not issame:
                 raise self.fspath.ImportMismatchError(modname, modfile, self)
-        except SyntaxError:
+        except SyntaxError as e:
             raise self.CollectError(
-                py.code.ExceptionInfo().getrepr(style="short"))
+                ExceptionInfo.from_current().getrepr(style="short")
+            ) from e
         except self.fspath.ImportMismatchError:
             e = sys.exc_info()[1]
             raise self.CollectError(
@@ -191,21 +187,37 @@ class OdooTestPackage(_pytest.python.Package, OdooTestModule):
         return "<Package %r>" % (getattr(self, "name", None), )
 
 
-def supports_from_parent(node):
-    """ Check if pytest supports from_parent constructor
-
-    Node Construction changed to Node.from_parent in pytest 5.4
-    """
-    if hasattr(node, "from_parent"):
-        return True
-
-
-def pytest_pycollect_makemodule(path, parent):
+def pytest_pycollect_makemodule(module_path, path, parent):
+    if not _find_manifest_path(module_path):
+        return None
     if path.basename == "__init__.py":
-        if supports_from_parent(OdooTestPackage):
-            return OdooTestPackage.from_parent(parent, fspath=path)
-        return OdooTestPackage(path, parent)
+        return OdooTestPackage.from_parent(parent, path=Path(path))
     else:
-        if supports_from_parent(OdooTestModule):
-            return OdooTestModule.from_parent(parent, fspath=path)
-        return OdooTestModule(path, parent)
+        return OdooTestModule.from_parent(parent, path=Path(path))
+
+
+def _find_manifest_path(collection_path: Path) -> Path:
+    """Try to locate an Odoo manifest file in the collection path."""
+    # check if collection_path is an addon directory
+    candidates = (
+        collection_path,  # addon directory
+        collection_path.parent,  # tests dir of an addon
+        collection_path.parent.parent  # file in the tests dir of an addon
+    )
+    for path in candidates:
+        manifest_path = path / "__manifest__.py"
+        if manifest_path.is_file():
+            return manifest_path
+    return None
+
+
+def pytest_ignore_collect(collection_path: Path) -> Optional[bool]:
+    """Do not collect tests of modules that are marked non installable."""
+    manifest_path = _find_manifest_path(collection_path)
+    if not manifest_path:
+        return None
+    manifest = ast.literal_eval(manifest_path.read_text())
+    if not manifest.get("installable", True):
+        # installable = False, do not collect this
+        return True
+    return None
